/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.l2jserver.gameserver.skills.conditions;

import com.l2jserver.gameserver.model.L2Effect;
import com.l2jserver.gameserver.skills.Env;

/**
 * The Class ConditionTargetActiveEffectId.
 */
public class ConditionTargetActiveEffectId extends Condition
{
    
    private final int _effectId;
    private final int _effectLvl;
    
    /**
     * Instantiates a new condition target active effect id.
     *
     * @param effectId the effect id
     */
    public ConditionTargetActiveEffectId(int effectId)
    {
        _effectId = effectId;
        _effectLvl = -1;
    }
    
    /**
     * Instantiates a new condition target active effect id.
     *
     * @param effectId the effect id
     * @param effectLevel the effect level
     */
    public ConditionTargetActiveEffectId(int effectId, int effectLevel)
    {
    	_effectId = effectId;
    	_effectLvl = effectLevel;
    }
    
    /* (non-Javadoc)
     * @see com.l2jserver.gameserver.skills.conditions.Condition#testImpl(com.l2jserver.gameserver.skills.Env)
     */
    @Override
    public boolean testImpl(Env env)
    {
    	final L2Effect e = env.target.getFirstEffect(_effectId);
    	if (e != null && (_effectLvl == -1 || _effectLvl <= e.getSkill().getLevel()))
    		return true;

    	return false;
    }
}
